@@ -18,9 +18,9 @@
 Authentication, error handling, etc are left as an exercise for the reader :)
 """
 
+import asyncio
 import logging
 import tornado.escape
-import tornado.ioloop
 import tornado.options
 import tornado.web
 import tornado.websocket
@@ -34,23 +34,21 @@ define("port", default=8888, help="run on the given port", type=int)
 
 class Application(tornado.web.Application):
     def __init__(self):
-        handlers = [
-            (r"/", MainHandler),
-            (r"/chatsocket", ChatSocketHandler),
-        ]
+        handlers = [(r"/", MainHandler), (r"/chatsocket", ChatSocketHandler)]
         settings = dict(
             cookie_secret="__TODO:_GENERATE_YOUR_OWN_RANDOM_VALUE_HERE__",
             template_path=os.path.join(os.path.dirname(__file__), "templates"),
             static_path=os.path.join(os.path.dirname(__file__), "static"),
             xsrf_cookies=True,
         )
-        super(Application, self).__init__(handlers, **settings)
+        super().__init__(handlers, **settings)
 
 
 class MainHandler(tornado.web.RequestHandler):
     def get(self):
         self.render("index.html", messages=ChatSocketHandler.cache)
 
+
 class ChatSocketHandler(tornado.websocket.WebSocketHandler):
     waiters = set()
     cache = []
@@ -70,7 +68,7 @@ class ChatSocketHandler(tornado.websocket.WebSocketHandler):
     def update_cache(cls, chat):
         cls.cache.append(chat)
         if len(cls.cache) > cls.cache_size:
-            cls.cache = cls.cache[-cls.cache_size:]
+            cls.cache = cls.cache[-cls.cache_size :]
 
     @classmethod
     def send_updates(cls, chat):
@@ -84,23 +82,21 @@ class ChatSocketHandler(tornado.websocket.WebSocketHandler):
     def on_message(self, message):
         logging.info("got message %r", message)
         parsed = tornado.escape.json_decode(message)
-        chat = {
-            "id": str(uuid.uuid4()),
-            "body": parsed["body"],
-            }
+        chat = {"id": str(uuid.uuid4()), "body": parsed["body"]}
         chat["html"] = tornado.escape.to_basestring(
-            self.render_string("message.html", message=chat))
+            self.render_string("message.html", message=chat)
+        )
 
         ChatSocketHandler.update_cache(chat)
         ChatSocketHandler.send_updates(chat)
 
 
-def main():
+async def main():
     tornado.options.parse_command_line()
     app = Application()
     app.listen(options.port)
-    tornado.ioloop.IOLoop.current().start()
+    await asyncio.Event().wait()
 
 
 if __name__ == "__main__":
-    main()
+    asyncio.run(main())
