@@ -8,27 +8,16 @@ apt-get update
 # has bugs that make our tests deadlock (the relevant tests detect this and
 # disable themselves, but it means that to get full coverage we have to use
 # the openssl version).
-# The oddly-named python-software-properties includes add-apt-repository.
 APT_PACKAGES="
 python-pip
 python-dev
+python3-pycurl
 libcurl4-openssl-dev
-python-software-properties
 "
 
 apt-get -y install $APT_PACKAGES
 
-
-# Ubuntu 10.04 has python 2.6 as default; install more from here.
-add-apt-repository ppa:fkrull/deadsnakes
-apt-get update
-
-DEADSNAKES_PACKAGES="
-python2.7
-python2.7-dev
-"
-apt-get -y install $DEADSNAKES_PACKAGES
-
+# Ubuntu 14.04 includes python 2.7 and 3.4.
 
 PIP_PACKAGES="
 futures
